-- License CC0 (Creative Commons license) (c) darkfrei, 2023


------------------------------
--------- background ---------
------------------------------

local function newBack (imageName, vx)
	local img = love.graphics.newImage(imageName)
	local bgr = {
		img = img,
		x = 0,
		vx = vx,
		w = img:getWidth()
	}
	return bgr
end

local function updateBack (bgr, dt)
	bgr.x = bgr.x + dt*bgr.vx
	if bgr.x < -bgr.w then 
		bgr.x = bgr.x + bgr.w
	elseif bgr.x > 0 then 
		bgr.x = bgr.x - bgr.w
	end
end

local function drawBack (bgr)
	love.graphics.setColor(1,1,1)
	local w = love.graphics.getWidth()
	for x = bgr.x, w, bgr.w do
		love.graphics.draw (bgr.img, math.floor(x+0.5), 0)
	end	
end


---------------------------
--------- buttons ---------
---------------------------

local function newButton (imgN, imgH, imgP, x,y, func, text)
	local normal = love.graphics.newImage(imgN)
	local hovered = love.graphics.newImage(imgH)
	local pressed = love.graphics.newImage(imgP)
	local w, h = normal:getDimensions()
	local font = love.graphics.newFont(0.4*h)
	local button = {
		x=x, y=y, w=w, h=h,
		images = {
			normal=normal,
			hovered=hovered,
			pressed=pressed
		},
		img = normal,
		pressed=false,
		hovered=false,
		text=text,
		textColor = {0,0,0},
		textShiftY = 0.5*h-0.5*font:getHeight(),
		font = font,
		func = func
	}
	button.tx=0
	button.ty=0
	return button
end

local function updateButton (button, pressed, released)
	local mx,my = love.mouse.getPosition()
	local x,y,w,h=button.x, button.y, button.w, button.h
	
	if mx>x and mx<(x+w) and my>y and my<y+h then
		-- mouse in the button area
		if not button.hovered then
			button.hovered = true
			-- event for hovered button here
		end
		if pressed then 
			-- the button was pressed right now
			button.pressed = true
			-- event for pressed (not released) button
		elseif button.pressed and released then
			-- the button was released right now
			button.pressed = false
			-- event to call button function here
			-- for example:
			button.func()
		end
	elseif released then -- not pressed anymore
		button.pressed = false
		-- event for released button here
	elseif button.hovered then -- not hovered anymore
		button.hovered = false
		-- event for unhovered button here
	end
	
	-- graphical preparations:
	if button.hovered and button.pressed then
		button.img = button.images.pressed
		button.tx, button.ty = 2, 2
	elseif button.hovered then
		button.img = button.images.hovered
		button.tx, button.ty = 0, 0
	else
		button.img = button.images.normal
		button.tx, button.ty = 0, 0
	end
	
	-- returns true if the mouse was on this button
	return button.hovered 
end

local function drawButton (button)
	love.graphics.setColor(1,1,1)
	love.graphics.draw(button.img, button.x, button.y)
	
	love.graphics.setFont(button.font)
	love.graphics.setColor(button.textColor)
	love.graphics.printf( button.text, button.x+button.tx, button.y+button.ty+button.textShiftY, button.w, "center")
end


------------------------
--------- love ---------
------------------------

function love.load()
	bgr1 = newBack ('back-1.png', -20)
	bgr2 = newBack ('back-2.png', -40)
	bgr3 = newBack ('back-3.png', -160)
	
	local imgN = 'button-normal.png'
	local imgH = 'button-hovered.png'
	local imgP = 'button-pressed.png'
	local funcExit = love.event.quit
	
	button=newButton (imgN, imgH, imgP, 600,480, funcExit, "Exit")
	
	local reverseDirection = function ()
		bgr1.vx = -bgr1.vx
		bgr2.vx = -bgr2.vx
		bgr3.vx = -bgr3.vx
	end
	button2=newButton (imgN, imgH, imgP, 400,480, reverseDirection, "Reverse")
end

function love.update(dt)
	if dt > 0.1 then dt = 0.1 end
	updateBack (bgr1, dt)
	updateBack (bgr2, dt)
	updateBack (bgr3, dt)
end

function love.draw()
	drawBack (bgr1)
	drawBack (bgr2)
	drawBack (bgr3)
	
	-- drawing later means drawing higher
	drawButton (button)
	drawButton (button2)
end

function love.keypressed(key, scancode, isrepeat)
	if false then
	elseif key == "escape" then
		love.event.quit()
	end
end

function love.mousepressed( x, y, mb, istouch, presses )
	if mb == 1 then -- left mouse button
		updateButton (button, true, false) -- button, pressed, released
		updateButton (button2, true, false) -- button, pressed, released
	elseif mb == 2 then -- right mouse button
	end
end

function love.mousemoved( x, y, dx, dy, istouch )
	updateButton (button, false, false) -- button, pressed, released
	updateButton (button2, false, false) -- button, pressed, released
end

function love.mousereleased( x, y, mb, istouch, presses )
	if mb == 1 then -- left mouse button
		updateButton (button, false, true) -- button, pressed, released
		updateButton (button2, false, true) -- button, pressed, released
	elseif mb == 2 then -- right mouse button
		
	end
end